#!/usr/bin/env sh
# shellcheck source=onedatify/onedatify

install_main() {

  install_admin_password=""
  install_oneprovider_version=""
  install_oneprovider_interactive=0

  if [ $(( $# )) -eq 0 ]; then
    install_usage
    exit 0
  fi

  while [ $(( $# )) -ne 0 ] ; do
      case $1 in
          -h|-\?|--help)   # Call a "usage" function to display a synopsis, then exit.
              install_usage
              exit 0
              ;;
          -p|--password)
              install_admin_password="$2"
              shift
              ;;
          -v|--oneprovider-version)
              install_oneprovider_version="$2"
              shift
              ;;
          interactive)
              install_oneprovider_interactive=1
              ;;
          -?*|*)
              printf 'WARN: Unknown option (ignored): %s\n' "$1" >&2
              exit 1
              ;;
      esac
      shift
  done

  if [ "$install_admin_password" = "" ]; then
    echo "ERROR. Please supply admin password. See -h."
    exit 1
  fi

  . "${onedatify_config_dir}/config.sh"
  if ! config_main validate 1; then
    echo "ERROR. Configuration is not complete or contains errors. Please run onedatify config to fix it"
    exit 1
  fi

  if [ "$install_oneprovider_version" != "" ]; then
    . "${onedatify_config_dir}/upgrade.sh"
    upgrade_retag "$config_version" "$install_oneprovider_version"
  fi

  if ! [ -f "$onedatify_docker_compose_path" ]; then
    cp "$onedatify_docker_compose_template_path" "$onedatify_docker_compose_path"
  fi

  if [ "$install_oneprovider_interactive" = "0" ]; then 
    install_and_run_oneprovider "$admin_password"
  fi
 
}

install_provider_ready_check() {
  user=""
  match="0"
  while [ "$deployed" != "true" ] ; do
    printf "."
    if command_exists curl; then
      deployed=$(curl -k -sS -X GET 'https://localhost:9443/api/v3/onepanel/configuration' 2>/dev/null | sed -n 's/.*"deployed":\(true\).*/\1/gp') ;
    elif command_exists wget; then
      deployed=$(wget -qO- --no-check-certificate 'https://localhost:9443/api/v3/onepanel/configuration' | sed -n 's/.*"deployed":\(true\).*/\1/gp') ;
    fi
    # $sh_c "systemctl -q is-active onedatify.service >/dev/null && systemctl stop onedatify.service"
    sleep 3
    if ! systemctl is-active --quiet onedatify ; then
      echo "" ; message_imp "ONEPROVIDER FAILED TO START!"
      message_imp "Please use command \"onedatify logs\" to inspect the cause of a failure. Exiting..."
      exit 1
    fi
  done
}

# TODO add timeout as descrubed here
# https://stackoverflow.com/questions/27555727/timeouting-a-while-loop-in-linux-bash-one-line-shell-scripting
install_provider_nagios_check() {
  while [ "$response" != "ok" ]; do
    response="$(curl -k --fail  --silent https://localhost/nagios | tr '>' '\n' | sed -n 's/<healthdata.* date.*status="\(ok\)"/\1/p')"
    sleep 0.5
  done
}

install_and_run_oneprovider() {
  admin_password=$1
  
  # Start in background
  $sh_c 'systemctl enable onedatify.service'
  $sh_c 'systemctl start onedatify.service'
  
  # Wait for ready
  message ''
  message "####### Deploying Oneprovider ########"
  message 'The deployment process is about to start, you can observe detailed logs by executing the following command in another window:'
  message '    onedatify logs'
  message ''
  _message 'Waiting for Oneprovider to start...'
  install_provider_ready_check
  message ''
  message 'Congratulations! Oneprovider has started successfully.'
  printf '\n\n'

}

install_usage() {
cat <<EOF
onedatify install -p <admin password>

  -p,--password   admin password

EOF
}

#install_main "$@"